@@ -104,7 +104,7 @@ static char *cursor_theme = NULL;
 static int   cursor_size = 24;
 static gboolean compositing_manager = FALSE;
 static gboolean resize_with_right_button = FALSE;
-static gboolean side_by_side_tiling = FALSE;
+static gboolean edge_tiling = FALSE;
 static gboolean force_fullscreen = TRUE;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_FULLSCREEN_FLASH;
@@ -423,9 +423,9 @@ static MetaBoolPreference preferences_bool[] =
       &resize_with_right_button,
       FALSE,
     },
-    { "/apps/metacity/general/side_by_side_tiling",
-      META_PREF_SIDE_BY_SIDE_TILING,
-      &side_by_side_tiling,
+    { "/apps/metacity/general/edge_tiling",
+      META_PREF_EDGE_TILING,
+      &edge_tiling,
       FALSE,
     },
     { "/apps/mutter/general/live_hidden_windows",
@@ -2010,8 +2010,8 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_RESIZE_WITH_RIGHT_BUTTON:
       return "RESIZE_WITH_RIGHT_BUTTON";
 
-    case META_PREF_SIDE_BY_SIDE_TILING:
-      return "SIDE_BY_SIDE_TILING";
+    case META_PREF_EDGE_TILING:
+      return "EDGE_TILING";
 
     case META_PREF_FORCE_FULLSCREEN:
       return "FORCE_FULLSCREEN";
@@ -2926,9 +2926,9 @@ meta_prefs_get_gnome_animations ()
 }
 
 gboolean
-meta_prefs_get_side_by_side_tiling ()
+meta_prefs_get_edge_tiling ()
 {
-  return side_by_side_tiling;
+  return edge_tiling;
 }
 
 MetaKeyBindingAction
