@@ -634,7 +634,7 @@ meta_preference_get_type (void)
         { META_PREF_CURSOR_SIZE, "META_PREF_CURSOR_SIZE", "cursor-size" },
         { META_PREF_COMPOSITING_MANAGER, "META_PREF_COMPOSITING_MANAGER", "compositing-manager" },
         { META_PREF_RESIZE_WITH_RIGHT_BUTTON, "META_PREF_RESIZE_WITH_RIGHT_BUTTON", "resize-with-right-button" },
-        { META_PREF_SIDE_BY_SIDE_TILING, "META_PREF_SIDE_BY_SIDE_TILING", "side-by-side-tiling" },
+        { META_PREF_EDGE_TILING, "META_PREF_EDGE_TILING", "edge-tiling" },
         { META_PREF_FORCE_FULLSCREEN, "META_PREF_FORCE_FULLSCREEN", "force-fullscreen" },
         { META_PREF_CLUTTER_PLUGINS, "META_PREF_CLUTTER_PLUGINS", "clutter-plugins" },
         { META_PREF_LIVE_HIDDEN_WINDOWS, "META_PREF_LIVE_HIDDEN_WINDOWS", "live-hidden-windows" },
