@@ -23,39 +23,29 @@
 #include "gradient.h"
 #include <gtk/gtk.h>
 
-typedef void (* RenderGradientFunc) (GdkDrawable *drawable,
-                                     GdkGC       *gc,
+typedef void (* RenderGradientFunc) (cairo_t     *cr,
                                      int          width,
                                      int          height);
 
 static void
-draw_checkerboard (GdkDrawable *drawable,
-                   int          width,
-                   int          height)
+draw_checkerboard (cairo_t *cr,
+                   int      width,
+                   int      height)
 {
   gint i, j, xcount, ycount;
-  GdkGC *gc1, *gc2;
-  GdkColor color;
+  GdkColor color1, color2;
   
 #define CHECK_SIZE 10
 #define SPACING 2  
 
-  /* It would be a bit more efficient to keep these
-   * GC's around instead of recreating on each expose, but
-   * this is the lazy/slow way.
-   */
-  gc1 = gdk_gc_new (drawable);
-  color.red = 30000;
-  color.green = 30000;
-  color.blue = 30000;
-  gdk_gc_set_rgb_fg_color (gc1, &color);
+  color1.red = 30000;
+  color1.green = 30000;
+  color1.blue = 30000;
+
+  color2.red = 50000;
+  color2.green = 50000;
+  color2.blue = 50000;
 
-  gc2 = gdk_gc_new (drawable);
-  color.red = 50000;
-  color.green = 50000;
-  color.blue = 50000;
-  gdk_gc_set_rgb_fg_color (gc2, &color);
-  
   xcount = 0;
   i = SPACING;
   while (i < width)
@@ -64,23 +54,17 @@ draw_checkerboard (GdkDrawable *drawable,
       ycount = xcount % 2; /* start with even/odd depending on row */
       while (j < height)
 	{
-	  GdkGC *gc;
-	  
 	  if (ycount % 2)
-	    gc = gc1;
+	    gdk_cairo_set_source_color (cr, &color1);
 	  else
-	    gc = gc2;
+	    gdk_cairo_set_source_color (cr, &color2);
 
 	  /* If we're outside event->area, this will do nothing.
 	   * It might be mildly more efficient if we handled
 	   * the clipping ourselves, but again we're feeling lazy.
 	   */
-	  gdk_draw_rectangle (drawable,
-			      gc,
-			      TRUE,
-			      i, j,
-			      CHECK_SIZE,
-			      CHECK_SIZE);
+          cairo_rectangle (cr, i, j, CHECK_SIZE, CHECK_SIZE);
+          cairo_fill (cr);
 
 	  j += CHECK_SIZE + SPACING;
 	  ++ycount;
@@ -89,14 +73,10 @@ draw_checkerboard (GdkDrawable *drawable,
       i += CHECK_SIZE + SPACING;
       ++xcount;
     }
-  
-  g_object_unref (G_OBJECT (gc1));
-  g_object_unref (G_OBJECT (gc2));
 }
 
 static void
-render_simple (GdkDrawable *drawable,
-               GdkGC       *gc,
+render_simple (cairo_t     *cr,
                int width, int height,
                MetaGradientType type,
                gboolean    with_alpha)
@@ -128,55 +108,46 @@ render_simple (GdkDrawable *drawable,
                                alphas, G_N_ELEMENTS (alphas),
                                META_GRADIENT_HORIZONTAL);
       
-      draw_checkerboard (drawable, width, height);
+      draw_checkerboard (cr , width, height);
     }
     
-  gdk_draw_pixbuf (drawable,
-                   gc,
-                   pixbuf,
-                   0, 0,
-                   0, 0, width, height,
-                   GDK_RGB_DITHER_MAX,
-                   0, 0);
+  gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+  cairo_rectangle (cr, 0, 0, width, height);
+  cairo_fill (cr);
 
   g_object_unref (G_OBJECT (pixbuf));
 }
 
 static void
-render_vertical_func (GdkDrawable *drawable,
-                      GdkGC *gc,
+render_vertical_func (cairo_t *cr,
                       int width, int height)
 {
-  render_simple (drawable, gc, width, height, META_GRADIENT_VERTICAL, FALSE);
+  render_simple (cr, width, height, META_GRADIENT_VERTICAL, FALSE);
 }
 
 static void
-render_horizontal_func (GdkDrawable *drawable,
-                        GdkGC *gc,
+render_horizontal_func (cairo_t *cr,
                         int width, int height)
 {
-  render_simple (drawable, gc, width, height, META_GRADIENT_HORIZONTAL, FALSE);
+  render_simple (cr, width, height, META_GRADIENT_HORIZONTAL, FALSE);
 }
 
 static void
-render_diagonal_func (GdkDrawable *drawable,
-                      GdkGC *gc,
+render_diagonal_func (cairo_t *cr,
                       int width, int height)
 {
-  render_simple (drawable, gc, width, height, META_GRADIENT_DIAGONAL, FALSE);
+  render_simple (cr, width, height, META_GRADIENT_DIAGONAL, FALSE);
 }
 
 static void
-render_diagonal_alpha_func (GdkDrawable *drawable,
-                            GdkGC *gc,
+render_diagonal_alpha_func (cairo_t *cr,
                             int width, int height)
 {
-  render_simple (drawable, gc, width, height, META_GRADIENT_DIAGONAL, TRUE);
+  render_simple (cr, width, height, META_GRADIENT_DIAGONAL, TRUE);
 }
 
 static void
-render_multi (GdkDrawable *drawable,
-              GdkGC       *gc,
+render_multi (cairo_t     *cr,
               int width, int height,
               MetaGradientType type)
 {
@@ -194,46 +165,37 @@ render_multi (GdkDrawable *drawable,
                                        colors, N_COLORS,
                                        type);
 
-  gdk_draw_pixbuf (drawable,
-		   gc,
-		   pixbuf,
-		   0, 0,
-		   0, 0,
-		   width, height,
-		   GDK_RGB_DITHER_NORMAL,
-		   0, 0);
+  gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+  cairo_rectangle (cr, 0, 0, width, height);
+  cairo_fill (cr);
 
   g_object_unref (G_OBJECT (pixbuf));
 #undef N_COLORS
 }
 
 static void
-render_vertical_multi_func (GdkDrawable *drawable,
-                            GdkGC *gc,
+render_vertical_multi_func (cairo_t *cr,
                             int width, int height)
 {
-  render_multi (drawable, gc, width, height, META_GRADIENT_VERTICAL);
+  render_multi (cr, width, height, META_GRADIENT_VERTICAL);
 }
 
 static void
-render_horizontal_multi_func (GdkDrawable *drawable,
-                              GdkGC *gc,
+render_horizontal_multi_func (cairo_t *cr,
                               int width, int height)
 {
-  render_multi (drawable, gc, width, height, META_GRADIENT_HORIZONTAL);
+  render_multi (cr, width, height, META_GRADIENT_HORIZONTAL);
 }
 
 static void
-render_diagonal_multi_func (GdkDrawable *drawable,
-                            GdkGC *gc,
+render_diagonal_multi_func (cairo_t *cr,
                             int width, int height)
 {
-  render_multi (drawable, gc, width, height, META_GRADIENT_DIAGONAL);
+  render_multi (cr, width, height, META_GRADIENT_DIAGONAL);
 }
 
 static void
-render_interwoven_func (GdkDrawable *drawable,
-                        GdkGC       *gc,
+render_interwoven_func (cairo_t *cr,
                         int width, int height)
 {
   GdkPixbuf *pixbuf;
@@ -249,36 +211,30 @@ render_interwoven_func (GdkDrawable *drawable,
                                             colors, height / 10,
                                             colors + 2, height / 14);
 
-  gdk_draw_pixbuf (drawable,
-                   gc,
-                   pixbuf,
-                   0, 0,
-                   0, 0,
-                   width, height,
-                   GDK_RGB_DITHER_NORMAL,
-                   0, 0);
+  gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+  cairo_rectangle (cr, 0, 0, width, height);
+  cairo_fill (cr);
 
   g_object_unref (G_OBJECT (pixbuf));
 }
 
 static gboolean
-expose_callback (GtkWidget *widget,
-                 GdkEventExpose *event,
-                 gpointer data)
+draw_callback (GtkWidget *widget,
+               cairo_t   *cr,
+               gpointer   data)
 {
   RenderGradientFunc func = data;
-  GtkAllocation allocation;
   GtkStyle *style;
 
   style = gtk_widget_get_style (widget);
-  gtk_widget_get_allocation (widget, &allocation);
 
-  (* func) (gtk_widget_get_window (widget),
-            style->fg_gc[gtk_widget_get_state (widget)],
-            allocation.width,
-            allocation.height);
+  gdk_cairo_set_source_color (cr, &style->fg[gtk_widget_get_state (widget)]);
 
-  return TRUE;
+  (* func) (cr,
+            gtk_widget_get_allocated_width (widget),
+            gtk_widget_get_allocated_height (widget));
+
+  return FALSE;
 }
 
 static GtkWidget*
@@ -299,8 +255,8 @@ create_gradient_window (const char *title,
   gtk_window_set_default_size (GTK_WINDOW (window), 175, 175);
   
   g_signal_connect (G_OBJECT (drawing_area),
-                    "expose_event",
-                    G_CALLBACK (expose_callback),
+                    "draw",
+                    G_CALLBACK (draw_callback),
                     func);
 
   gtk_container_add (GTK_CONTAINER (window), drawing_area);
